@@ -298,7 +298,7 @@ read_w32_registry_string (const char *root, const char *dir, const char *name)
 char *
 get_data_dir (void)
 {
-  char *instdir;
+  const char *instdir;
   char *p;
   char *dname;
 
@@ -310,17 +310,12 @@ get_data_dir (void)
 #define SDDIR "\\share\\gpgol"
   dname = (char*) xmalloc (strlen (instdir) + strlen (SDDIR) + 1);
   if (!dname)
-    {
-      xfree (instdir);
-      return NULL;
-    }
+    return NULL;
   p = dname;
   strcpy (p, instdir);
   p += strlen (instdir);
   strcpy (p, SDDIR);
 
-  xfree (instdir);
-
 #undef SDDIR
   return dname;
 }
@@ -581,8 +576,74 @@ wchar_t*
 get_tmp_outfile (const wchar_t *name, HANDLE *outHandle)
 {
   TSTART
-  auto utf8Name = sanitizeFileName(wchar_to_utf8_string (name));
   const auto tmpPath = getTmpPathUtf8 ();
+  auto utf8Name = sanitizeFileName(wchar_to_utf8_string (name));
+
+  auto fileExtension = wchar_to_utf8_string (name);
+  auto fileExtPos = fileExtension.rfind('.');
+  if (fileExtPos != std::string::npos)
+  {
+    fileExtension = fileExtension.substr(fileExtPos);
+    // truncate in case we don't have a real extension and just a point in a filename
+    if (fileExtension.size() > 10)
+      {
+        fileExtension.erase(10);
+      }
+
+    log_dbg("%s:%s: Extension found %s",
+            SRCNAME, __func__, fileExtension.c_str() );
+
+    // check if the file extension was modified by sanitize ...
+    fileExtPos = utf8Name.rfind(fileExtension);
+    if (fileExtPos==std::string::npos)
+      {
+        fileExtPos = utf8Name.rfind(fileExtension.substr(1)); // try again but skip . in fileExt
+      }
+
+    if (fileExtPos != std::string::npos)
+      {
+        utf8Name.erase(fileExtPos);
+        utf8Name.append(fileExtension);
+      }
+    else
+      {
+        // should not happen
+        log_error("%s:%s: Extension %s not found in %s",
+               SRCNAME, __func__, fileExtension.c_str(), utf8Name.c_str());
+      }
+  }
+  else
+  {
+    log_dbg("%s:%s: No fileextension found",
+                SRCNAME, __func__);
+    // no extension
+    // utf8Name.append(".tmp");
+  }
+
+  if( tmpPath.size() + utf8Name.size() > MAX_PATH - 1)
+    {
+        log_dbg("%s:%s: MAX_PATH limit failed %s%s",
+                    SRCNAME, __func__, tmpPath.c_str(), utf8Name.c_str ());
+        fileExtPos = utf8Name.rfind('.');
+        fileExtension.clear();
+        if (fileExtPos != std::string::npos)
+          {
+            fileExtension = utf8Name.substr(fileExtPos);
+            // truncate in case we don't have a real extension and just a point in a filename
+            log_dbg("%s:%s: Extension found %s",
+                    SRCNAME, __func__, fileExtension.c_str() );
+          }
+      utf8Name.erase( MAX_PATH - ( tmpPath.size() + fileExtension.size() ));
+      utf8Name.append(fileExtension);
+      log_dbg("%s:%s: changed to %s%s",
+              SRCNAME, __func__, tmpPath.c_str(), utf8Name.c_str ());
+    }
+  else
+    {
+      log_dbg("%s:%s: MAX_PATH check passed %lld <= %d",
+              SRCNAME, __func__,
+              (long long int)tmpPath.size() + utf8Name.size() , MAX_PATH - 1);
+    }
 
   if (utf8Name.empty() || tmpPath.empty())
     {
@@ -646,7 +707,11 @@ get_tmp_outfile (const wchar_t *name, HANDLE *outHandle)
       // OutNameC is now without an extension and if
       // there is a file ext it now points to the extension.
 
-      outName = outNameC + std::string("_") + std::to_string(tries++);
+      auto numExt = std::string("_") + std::to_string(tries++);
+
+      outName = outNameC;
+      outName.erase(outName.end() - numExt.size());
+      outName.append(numExt);
 
       if (fileExt)
         {
@@ -703,36 +768,59 @@ get_tmp_outfile_utf8 (const char *name, HANDLE *outHandle)
 
 /** Get the Gpg4win Install directory.
  *
- * Looks first for the Gpg4win 3.x registry key. Then for the Gpg4win
- * 2.x registry key. And checks that the directory can be read.
+ * This function returns the root of install directory of Gpg4win or
+ * GnuPG-[VS-]Desktop.
  *
- * @returns NULL if no dir could be found. Otherwise a malloced string.
+ * @returns NULL if no dir could be found - this indoicates a bad
+ * installation.  Otherwise a malloced string with the utf-8 name.
  */
-char *
+const char *
 get_gpg4win_dir()
 {
-  const char *g4win_keys[] = {GPG4WIN_REGKEY_3,
-                              GPG4WIN_REGKEY_2,
-                              NULL};
-  const char **key;
-  for (key = g4win_keys; *key; key++)
+  static int initialized;
+  static char *mydir;
+
+  if (!initialized)
     {
-      char *tmp = read_w32_registry_string (NULL, *key, "Install Directory");
-      if (!tmp)
-        {
-          continue;
-        }
-      if (!access(tmp, R_OK))
-        {
-          return tmp;
-        }
+      wchar_t *wmodulename;
+      char *modulename = NULL;
+      char *p;
+
+
+      wmodulename = (wchar_t*)xcalloc (MAX_PATH+5, sizeof *wmodulename);
+      if (!GetModuleFileNameW (glob_hinst, wmodulename, MAX_PATH))
+        log_debug ("GetModuleFileName failed\n");
       else
         {
-          log_debug ("Failed to access: %s\n", tmp);
-          xfree (tmp);
+          modulename = wchar_to_utf8 (wmodulename);
+          log_debug ("GetModuleFileName: '%s'\n", modulename);
+          p = strrchr (modulename, '\\');
+          if (p)
+            {
+              *p = 0;
+              /* MODULENAME is now the actual directory from where we
+               * were executed.  In the most cases this is either the
+               * bin or the bin_64 sub directory.  We tell gpgme that
+               * directory so that it can make use of it.  This is
+               * required because gpgme is linked statically and can't
+               * find its installation directory.  */
+              gpgme_set_global_flag ("w32-inst-dir", modulename);
+              p = strrchr (modulename, '\\');
+              if (p)
+                {
+                  *p = 0;
+                  mydir = modulename;  /* Not really threadsafe, but okay.  */
+                  modulename = NULL;
+                  initialized = 1;
+                  log_debug ("Using install dir: '%s'\n", mydir);
+                }
+            }
         }
+      xfree (modulename);
+      xfree (wmodulename);
     }
-  return NULL;
+
+  return mydir;
 }
 
 
@@ -775,7 +863,8 @@ char *
 get_uiserver_name (void)
 {
   char *name = NULL;
-  char *dir, *uiserver, *p;
+  const char *dir;
+  char *uiserver, *p;
   int extra_arglen = 9;
 
   const char * server_names[] = {"kleopatra.exe",
@@ -810,7 +899,6 @@ get_uiserver_name (void)
   if (name && !access (name, F_OK))
     {
       /* Set through registry and is accessible */
-      xfree(dir);
       return name;
     }
   /* Fallbacks */
@@ -832,11 +920,9 @@ get_uiserver_name (void)
             {
               strcat (name, " --daemon");
             }
-          xfree (dir);
           return name;
         }
     }
-  xfree (dir);
   log_error ("Failed to find a viable UIServer");
   return NULL;
 }
@@ -1102,7 +1188,7 @@ de_vs_name (bool isCompliant)
   compName = utf8_gettext ("VS-NfD compliant");
   uncompName = utf8_gettext ("not VS-NfD compliant");
   /* Find the libkleopatrarc */
-  char *instdir = get_gpg4win_dir();
+  const char *instdir = get_gpg4win_dir();
   if (!instdir)
     {
       STRANGEPOINT;
@@ -1206,34 +1292,6 @@ compliance_string (bool forVerify, bool forDecrypt, bool isCompliant)
   return std::string();
 }
 
-char *
-get_gpgme_w32_inst_dir (void)
-{
-  char *gpg4win_dir = get_gpg4win_dir ();
-  char *tmp;
-  gpgrt_asprintf (&tmp, "%s\\bin\\gpgme-w32spawn.exe", gpg4win_dir);
-  memdbg_alloc (tmp);
-
-  if (!access(tmp, R_OK))
-    {
-      xfree (tmp);
-      gpgrt_asprintf (&tmp, "%s\\bin", gpg4win_dir);
-      memdbg_alloc (tmp);
-      xfree (gpg4win_dir);
-      return tmp;
-    }
-  xfree (tmp);
-  gpgrt_asprintf (&tmp, "%s\\gpgme-w32spawn.exe", gpg4win_dir);
-  memdbg_alloc (tmp);
-
-  if (!access(tmp, R_OK))
-    {
-      xfree (tmp);
-      return gpg4win_dir;
-    }
-  OutputDebugString("Failed to find gpgme-w32spawn.exe!");
-  return NULL;
-}
 
 #define WINDOWS_DEVICES_PATTERN "(CON|AUX|PRN|NUL|COM[1-9]|LPT[1-9])(\\..*)?"
 #define SLASHES "/\\"
